﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using static System.Runtime.InteropServices.JavaScript.JSType;
using System.Numerics;
using System.Windows.Markup;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucTransporteTTop.xaml
    /// </summary>
    /// 
    
    public class osTransporteTTop : osBase
    {
        private string _nombre = "Transporte TTOP";

        private float frictionCoefficient;
        private float velMax50hz;  // en metros por minuto      
        private float tiempoRampa;
        private bool esMarcha;

        private Rectangle Data = new Rectangle();

        public override float LeftPixels
        {
            get => PixelToMeter.Instance.calc.MetersToPixels(Data.Left);
            set
            {
                Data.Left = PixelToMeter.Instance.calc.PixelsToMeters(value);
                if (_visualRepresentation != null )
                    Canvas.SetLeft(_visualRepresentation,value);
                OnPropertyChanged(nameof(LeftPixels));
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float TopPixels
        {
            get => PixelToMeter.Instance.calc.MetersToPixels(Data.Top);
            set
            {
                Data.Top = PixelToMeter.Instance.calc.PixelsToMeters(value);
                if (_visualRepresentation != null)
                    Canvas.SetTop(_visualRepresentation, value);
                OnPropertyChanged(nameof(TopPixels));
                OnPropertyChanged(nameof(Top));
            }
        }
        public override float Left
        {
            get => Data.Left;
            set
            {
                Data.Left = value;
                if (_visualRepresentation != null)
                    Canvas.SetLeft(_visualRepresentation, PixelToMeter.Instance.calc.MetersToPixels(value));
                OnPropertyChanged(nameof(LeftPixels));
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => Data.Top;
            set
            {
                Data.Top = value;
                if (_visualRepresentation != null)
                    Canvas.SetTop(_visualRepresentation, PixelToMeter.Instance.calc.MetersToPixels(value));
                OnPropertyChanged(nameof(TopPixels));
                OnPropertyChanged(nameof(Top));
            }
        }


        public float Ancho {
            get => Data.Length;
            set
            {
                Data.Length = value;
                OnPropertyChanged(nameof(Ancho));
            }
        }
        public float Alto {
            get => Data.Width;
            set
            {
                Data.Width = value;
                OnPropertyChanged(nameof(Alto));
            }
        }
        public float AnchoPixels
        {
            get => (float)PixelToMeter.Instance.calc.MetersToPixels(Data.Length);
            set
            {
                    Data.Length = (float)PixelToMeter.Instance.calc.PixelsToMeters(value);
                    OnPropertyChanged(nameof(Ancho));
            }
        }
        public float AltoPixels
        {
            get => (float)PixelToMeter.Instance.calc.MetersToPixels(Data.Width);
            set
            {
                    Data.Width = (float)PixelToMeter.Instance.calc.PixelsToMeters(value);
                    OnPropertyChanged(nameof(Alto));
            }
        }

        public float Angulo
        {
            get => Data.Angle;
            set
            {
                Data.Angle = value; 
                OnPropertyChanged(nameof(Angulo));
            }
        }
        public float VelocidadActual
        {
            get => Data.Speed;
            set {
                Data.Speed = value;
                OnPropertyChanged(nameof(VelocidadActual));
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }

        public float FrictionCoefficient { get => frictionCoefficient; set => frictionCoefficient = value; }
        public float VelMax50hz { get => velMax50hz; set => velMax50hz = value; }
        public float TiempoRampa { get => tiempoRampa; set => tiempoRampa = value; }
        public bool EsMarcha { get => esMarcha; set => esMarcha = value; }

        public osTransporteTTop()
        {
            AnchoPixels = 100;
            AltoPixels = 10;
        }

        public override void ConnectSimManager(SimulationManager simulationManager)
        {
            simulationManager.rectangles.Add(Data);
        }
        public override void UpdateControl()
        {
        }

    }

    public partial class ucTransporteTTop : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucTransporteTTop()
        {
            InitializeComponent();
        }
        public void Resize(float width, float height)
        {
            if (Datos is osTransporteTTop datos)
                datos.Ancho = width;
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.LeftPixels = LeftPixels;
                Datos.TopPixels = TopPixels;
            }
        }
        public void Rotate(float Angle) { 
            if (Datos != null)
                if (Datos is osTransporteTTop datos)
                    datos.Angulo = Angle;
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 1;
        }

    }
}



